<?php

namespace App\Http\Controllers\admin;

use Carbon\Carbon;
use App\Models\User;
use Pest\Support\Str;
use App\Models\Invoice;
use App\Models\Contract;
use App\Mail\SendInvoice;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Mail;

class ContractsController extends Controller
{
    public function index()
    {
        $contracts = Contract::with('user')->latest()->paginate(10);
        return view('admin.contracts.index', compact('contracts'));
    }

    public function createContract()
    {
        $users = User::where('status', 'active')->where('role', 'user')->get();
        return view('admin.contracts.create', compact('users'));
    }

    public function storeContract(Request $request)
    {
        $validated = $request->validate([
            'user_id' => 'required|exists:users,id',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after:start_date',
            'payment_plan' => 'required|in:weekly,monthly',
            'daily_rate' => 'required|numeric|min:0',
            'amount' => 'required|numeric|min:0',
            'terms_conditions' => 'nullable|string'
        ]);

        DB::beginTransaction();
        try {
            // Create contract
            $validated['status'] = 'active';
            $contract = Contract::create($validated);

            // Generate invoices based on payment plan
            $this->generateInvoices($contract);

            DB::commit();

            return redirect()->route('admin.contracts')
                ->with('success', 'Contract created successfully');

        } catch (\Exception $e) {
            DB::rollBack();
            return redirect()->back()->withInput()
                ->with('error', 'Error creating contract: ' . $e->getMessage());
        }
    }

    private function generateInvoices($contract)
    {
        $startDate = Carbon::parse($contract->start_date);
        $endDate = Carbon::parse($contract->end_date);
        $currentDate = $startDate->copy();
        $invoiceNumber = 1;

        // Calculate number of payments
        $totalDays = $startDate->diffInDays($endDate);
        $totalAmount = $contract->amount;
        
        if ($contract->payment_plan === 'weekly') {
            $numberOfPayments = ceil($totalDays / 7);
            $paymentAmount = $totalAmount / $numberOfPayments;
            $interval = 'week';
        } else { // monthly
            $numberOfPayments = ceil($totalDays / 30);
            $paymentAmount = $totalAmount / $numberOfPayments;
            $interval = 'month';
        }

        // Generate invoices
        for ($i = 0; $i < $numberOfPayments; $i++) {
            $invoiceDate = Carbon::now();
            $dueDate = $currentDate->copy()->addDays(7); // Add 7 days buffer for payment

            Invoice::create([
                'contract_id' => $contract->id,
                'user_id' => $contract->user_id,
                'invoice_date' => $invoiceDate,
                'invoice_number' => 'INV-' . date('Y') . '-' . str_pad($invoiceNumber, 4, '0', STR_PAD_LEFT),
                'amount' => $paymentAmount,
                'tax_amount' => $paymentAmount, // Adjust this according to your tax calculation logic
                'due_date' => $dueDate,
                'status' => 'unpaid'
            ]);

            $currentDate->add(1, $interval);
            $invoiceNumber++;
        }
    }

    public function showContract($id)
    {
        $contract = Contract::with(['user', 'invoices'])->findOrFail($id);
        $contract->load(['user', 'invoices']);

        // Get invoices due this week
        $startOfWeek = Carbon::now()->startOfWeek();
        $endOfWeek = Carbon::now()->endOfWeek();

        $dueThisWeek = $contract->invoices()
            ->whereBetween('due_date', [$startOfWeek, $endOfWeek])
            ->where('status', 'unpaid')
            ->get();

        return view('admin.contracts.show', compact('contract', 'dueThisWeek'));
    }

    public function editContract($id)
    {
        // Retrieve the contract with related user and invoices
        $contract = Contract::with(['user', 'invoices'])->findOrFail($id);

        // Retrieve active users for the dropdown
        $users = User::where('status', 'active')->get();
        return view('admin.contracts.edit', compact('contract', 'users'));
    }

    public function updateContract(Request $request, $id)
    {
        $validated = $request->validate([
            'status' => 'required|in:active,inactive,expired',
            'terms_conditions' => 'nullable|string'
        ]);
        $contract = Contract::with(['user', 'invoices'])->findOrFail($id);
        $contract->update($validated);

        return redirect()->route('admin.contracts')
            ->with('success', 'Contract updated successfully');
    }

    public function deleteContract($id)
    {
        $contract = Contract::findOrFail($id);
        $contract->invoices()->delete();
        $contract->delete();

        return redirect()->route('admin.contracts')
            ->with('success', 'Contract deleted successfully');
    }

    public function showInvoice($id)
    {
        $invoice = Invoice::with(['contract', 'user'])->findOrFail($id);
        return view('admin.invoices.show', compact('invoice'));
    }

    public function editInvoice($id)
    {
        $invoice = Invoice::with(['contract', 'user'])->findOrFail($id);
        return view('admin.invoices.edit', compact('invoice'));
    }

    public function updateInvoice(Request $request, $id)
    {
        $validated = $request->validate([
            'status' => 'required|in:paid,unpaid'
        ]);
        $invoice = Invoice::findOrFail($id);
        $invoice->update($validated);

        return redirect()->route('admin.invoices')
            ->with('success', 'Invoice updated successfully');
    }

    public function deleteInvoice($id)
    {
        $invoice = Invoice::findOrFail($id);
        $invoice->delete();

        return redirect()->route('admin.invoices')
            ->with('success', 'Invoice deleted successfully');
    }

    public function sendInvoice($id)
    {
        try {
            $invoice = Invoice::with(['user', 'contract'])->findOrFail($id);
            
            // Check if user has email
            if (!$invoice->user->email) {
                return redirect()->back()
                    ->with('error', 'User does not have an email address.');
            }

            $invoice->payment_token = Str::random(32);

            // Send the email
            Mail::to($invoice->user->email)->send(new SendInvoice($invoice));

            // Update the invoice status to indicate it was sent
            $invoice->email_sent = true;
            $invoice->email_sent_at = now();
            $invoice->save();

            return redirect()->back()
                ->with('success', 'Invoice sent successfully to ' . $invoice->user->email);

        } catch (\Exception $e) {
            return redirect()->back()
                ->with('error', 'Failed to send invoice: ' . $e->getMessage());
        }
    }
}
