<?php

namespace App\Http\Controllers\front;

use Carbon\Carbon;
use Stripe\Stripe;
use App\Models\User;
use App\Models\Invoice;
use App\Models\Payment;
use Stripe\PaymentIntent;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Validator;

class PaymentsController extends Controller
{
    public function paymentPage($id)
    {
        $invoice = Invoice::with(['user', 'contract'])->findOrFail($id);
        Stripe::setApiKey(env('STRIPE_SECRET'));

        try {
            $paymentIntent = \Stripe\PaymentIntent::create([
                'amount' => $invoice->amount * 100, // Convert amount to cents
                'currency' => 'usd',
                'metadata' => [
                    'invoice_id' => $invoice->id,
                ],
            ]);
            $clientSecret = $paymentIntent->client_secret;

            return view('front.user.payments.payment', compact('invoice', 'clientSecret'));
        } catch (\Exception $e) {
            return redirect()->back()->with(['error' => $e->getMessage()], 500);
        }
        
    }

    public function handleStripeSuccess(Request $request)
    {
        // dd($request->all());

        Stripe::setApiKey(env('STRIPE_SECRET'));

        // Validate incoming request
        $validator = Validator::make($request->all(), [
            'payment_intent_id' => 'required|string',
            'invoice_id' => 'required|exists:invoices,id',
            'payment_amount' => 'required|numeric',
        ]);

        if ($validator->fails()) {
            return response()->json(['success' => false, 'message' => $validator->errors()->first()], 400);
        }

        try {
            $paymentIntentId = $request->payment_intent_id;

            // Retrieve the payment intent from Stripe
            $paymentIntent = PaymentIntent::retrieve($paymentIntentId);
            
            // Extract the transaction ID from the first charge
            $transactionId = $paymentIntent->latest_charge;
            // dd($transactionId);
            if (!$transactionId) {
                throw new \Exception('Transaction ID could not be retrieved from Stripe.');
            }

            $invoiceId = $request->invoice_id;
            $amount = $request->payment_amount;

            // Update the invoice
            $invoice = Invoice::findOrFail($invoiceId);
            $invoice->update([
                'status' => 'paid',
                'payment_method' => 'stripe',
                'payment_reference' => $transactionId,
                'paid_at' => Carbon::now(),
            ]);

            // Store payment details
            Payment::create([
                'user_id' => auth()->user()->id,
                'invoice_id' => $invoiceId,
                'payment_method' => 'stripe',
                'payment_status' => 'paid',
                'payment_amount' => $amount,
                'transaction_id' => $transactionId,
                'payment_meta' => json_encode(['stripe_data' => [
                    'payment_intent_id' => $paymentIntentId,
                    'transaction_id' => $transactionId,
                    'payment_amount' => $amount,
                ]]),
            ]);

            return response()->json(['success' => true, 'message' => 'Payment recorded successfully.', 'invoice' => $invoice]);

        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()], 500);
        }
    }

    public function handlePaypalSuccess(Request $request)
    {
        // dd($request->all());
        $request->validate([
            'invoice_id' => 'required|exists:invoices,id',
            'transaction_id' => 'required|string',
            'payment_amount' => 'required|numeric',
        ]);

        $invoiceId = $request->invoice_id;
        $transactionId = $request->transaction_id;
        $amount = $request->payment_amount;

        $paypal_data = [
            'invoice_id' => $invoiceId,
            'payment_amount' => $amount,
            'transaction_id' => $transactionId,
        ];

        // Update the invoice
        $invoice = Invoice::findOrFail($invoiceId);
        $invoice->update([
            'status' => 'paid',
            'payment_method' => 'paypal',
            'payment_reference' => $transactionId,
            'paid_at' => Carbon::now(),
        ]);

        // Store payment details
        Payment::create([
            'user_id' => auth()->user()->id,
            'invoice_id' => $invoiceId,
            'payment_method' => 'paypal',
            'payment_status' => 'paid',
            'payment_amount' => $amount,
            'transaction_id' => $transactionId,
            'payment_meta' => json_encode(['paypal_data' => $paypal_data]),
        ]);

        return response()->json(['success' => true, 'message' => 'Payment recorded successfully.', 'invoice' => $invoice]);
    }

    public function paymentReturn()
    {
        return view('front.user.payments.payment-return');
    }

    public function paymentDone(Invoice $invoice)
    {
        $invoice = Invoice::find($invoice->id);
        // dd($invoice);
        return view('front.user.payments.payment-done', compact('invoice'));
    }
}
